--Ŀ
-- Vec-Font.E V1.01 (C) 1997 PJB Systems               Vectored Font Library 
--
-- History
-- [100] 29-06-1997 Created. Released to RDS.
-- [101] 30-06-1997 Added crude line width

include Graphics.E
include Vec-Font.VF -- Default Font

global integer VFontXscale      VFontXscale = 2
global integer VFontYscale      VFontYscale = 2
global integer VFontHight       VFontHight = 10
global integer VFontLWidth      VFontLWidth = 1
global atom    VFontAngleX      VFontAngleX = 0
global atom    VFontAngleY      VFontAngleY = 0
global atom    VFontAngleZ      VFontAngleZ = 0

global integer VFontColour      VFontColour = 15
global integer VFontBackGd      VFontBackGd = 4
global integer VFontCrsrX       VFontCrsrX = 6
global integer VFontCrsrY       VFontCrsrY = 6

integer  x,y,z -- Temp
sequence LookUpTableX       LookUpTableX = repeat(0,99)
sequence LookUpTableY       LookUpTableY = repeat(0,99)
sequence CurrentFont        CurrentFont = VecFont -- Defined in Vec-Font.VF

-- Generate lookup table (Makes things go a little faster !)
for LuX = 1 to 7 do
  for LuY = 1 to 8 do
    z = (LuX * 10) + LuY
    LookUpTableX[z] = floor(z / 10) - 4
    LookUpTableY[z] = remainder(z,10) - 4
  end for
end for

-- This plots char C at co-ords Vx, Vy
global procedure PlotVChar(integer Vx,integer Vy,integer c)
  object    vfnt
  sequence  vtx
  integer   vx,vy,ix,iy,pix,lw,sl
  integer   x1,x2,x3,x4,y1,y2,y3,y4
  atom      ang
  atom      CosAx,SinAx,CosAy,SinAy,CosAz,SinAz
  
  pix = VFontColour

  ang = VFontAngleZ * -0.017453292 -- Pre-calculate Radians 2 Degrees
  CosAz = cos(ang) -- Pre-calculate Cosine(Az)
  SinAz = sin(ang) -- Pre-calculate Sine(Az)
  
  lw = VFontLWidth - 1
  if (c > 0) and (c < 255) then -- Sensible value for C
    vfnt = CurrentFont[c] -- Extract specific VFont data for C
    if sequence(vfnt) > 0 then -- 
      for e = 1 to length(vfnt) do -- Scan elements in font
	vtx = {} -- Clear vertices
	for i = 1 to length(vfnt[e]) do -- Scan element
	  z = vfnt[e][i]
	  ix = LookUpTableX[z]
	  iy = LookUpTableY[z]
	  vx = (ix * VFontXscale)
	  vy = (iy * VFontYscale)
	  x = floor((CosAz * vx) + (SinAz * vy)) -- Rotate about Az
	  y = floor((CosAz * vy) - (SinAz * vx))
	  vtx = append(vtx,{Vx + x , Vy + y})
	end for
	
	if length(vtx) > 1 then -- Polyline
	  if VFontLWidth < 2 then -- 1 pixel thick
	    draw_line(pix,vtx)
	  else -- 2+ pixels thick
	    x1 = vtx[1][1]
	    y1 = vtx[1][2]
	    sl = length(vtx)
	    for l = 2 to sl do
	      x2 = vtx[l][1]
	      y2 = vtx[l][2]
	      
	      for zz = -lw to lw do
		draw_line(pix,{{x1 + zz,y1},{x2 + zz,y2}})
		draw_line(pix,{{x1,y1 + zz},{x2,y2 + zz}})
	      end for
	      x3 = x1 - lw    y3 = y1 - lw
	      x4 = x1 + lw    y4 = y1 + lw
	      
	      ellipse(pix,1,{x3,y3},{x4,y4})
	      
	      x1 = x2
	      y1 = y2
	    end for
	    
	    x3 = x1 - lw    y3 = y1 - lw
	    x4 = x1 + lw    y4 = y1 + lw
	    ellipse(pix,1,{x3,y3},{x4,y4})
	    
	  end if
	elsif length(vtx) = 1 then -- Single dot
	  if VFontLWidth < 2 then -- 1 pixel thick
	    pixel(pix,vtx[1])
	  else -- Circle
	    x3 = vtx[1][1] - lw   y3 = vtx[1][2] - lw
	    x4 = vtx[1][1] + lw   y4 = vtx[1][2] + lw
	    ellipse(pix,1,{x3,y3},{x4,y4})
	  end if
	end if
      end for
    end if
  end if
end procedure

-- This prints string Txt at VFontCrsrX,VFontCrsrY in colour VFontColour
global procedure PrintVText(sequence Txt)
  integer c,width,hight,xl,xr,yt,yb,yc,l
  sequence vc
  
  vc = video_config()
  width = vc[5]
  hight = vc[6]
  l = length(Txt)
  
  for t = 1 to length(Txt) do
    c = Txt[t]
    xl = VFontCrsrX - (5 * VFontXscale)
    xr = VFontCrsrX + (5 * VFontXscale)
    yc = floor((VFontHight * VFontYscale) / 2)
    yt = VFontCrsrY - yc
    yb = VFontCrsrY + yc
    
    for yy = yt to yb do
      draw_line(VFontBackGd,{{xl,yy},{xr,yy}})
    end for
    
    PlotVChar(VFontCrsrX,VFontCrsrY,c)
    VFontCrsrX = VFontCrsrX + (10 * VFontXscale) -- Advance cursor
    if VFontCrsrX > width then -- Start new line
      VFontCrsrX = (5 * VFontXscale)
      VFontCrsrY = VFontCrsrY + (10 * VFontYscale)
    end if
    if VFontCrsrY > hight then -- Start at top of screen
      VFontCrsrX = (5 * VFontXscale)
      VFontCrsrY = (5 * VFontYscale)
    end if
  end for
end procedure


